
# coding: utf-8

# # Metadata

# In[ ]:


from .base_object import BaseObject


# In[ ]:


class Metadata( BaseObject ):
    """
    
    """
    
    def __init__( self ):
        """
        
        """
        pass

